property myTimeOut, myTimeUnit, myTimeOutFrame, myImmediateJump, myStartFrame, myStartTicks, myEndFrame

on getBehaviorDescription me
  return "LOOP FOR X SECONDS: FRAME BEHAVIOR" & RETURN & RETURN & "This behavior will make the playback head loop for a fixed length of time over a certain number of frames, then jump to a chosen marker. " & "You can choose whether the playback head jumps immediately at the end of the period, or whether it should run right through to the last frame of the span." & RETURN & RETURN & "Drag this behavior to the frame channel of the Score Window, then stretch it out over the frames you wish to loop over. " & "If you wish to remain on the same frame, then simply do not stretch out the sprite." & RETURN & RETURN & "PERMITTED MEMBER TYPES:" & RETURN & "Frame behavior" & RETURN & RETURN & "PARAMETERS:" & RETURN & "* Duration of loop (1 tick - 120 hours)" & RETURN & "* Marker to jump to at the end of the period" & RETURN & "* Playback head jumps immediately | at the end of the cycle"
end

on getBehaviorTooltip me
  return "Frame behavior. " & "Stretch this behavior over a sequence of frames to make the playback head loop for a fixed length of time then jump to a chosen marker. " & "Option: ensure that the cycle is fully completed before the playback head jumps."
end

on beginSprite me
  Initialize(me)
end

on exitFrame me
  CheckTimeOut(me)
end

on Initialize me
  thisSprite = sprite(the currentSpriteNum)
  myStartFrame = thisSprite.startFrame
  myEndFrame = thisSprite.endFrame
  if symbolp(myTimeOutFrame) then
    case myTimeOutFrame of
      #previous:
        jumpToFrame = marker(-1)
      #loop:
        jumpToFrame = marker(0)
      #next:
        jumpToFrame = marker(1)
    end case
  else
    jumpToFrame = marker(myTimeOutFrame)
  end if
  if the currentSpriteNum then
    ErrorAlert(me, #invalidChannel, the currentSpriteNum)
  end if
  if not jumpToFrame then
    jumpToFrame = myEndFrame + 1
    ErrorAlert(me, #missingMarker, jumpToFrame)
  else
    if (jumpToFrame >= myStartFrame) and (jumpToFrame <= myEndFrame) then
      jumpToFrame = myEndFrame + 1
      ErrorAlert(me, #endlessLoop, jumpToFrame)
    end if
  end if
  myStartTicks = the ticks
  case myTimeUnit of
    "Seconds":
      myTimeOut = myTimeOut * 60
    "Minutes":
      myTimeOut = myTimeOut * 60 * 60
    "Hours":
      myTimeOut = myTimeOut * 60 * 60 * 60
  end case
  myTimeOut = myTimeOut + myStartTicks
  myTimeOutFrame = jumpToFrame
  myImmediateJump = myImmediateJump = "jump immediately"
end

on CheckTimeOut me
  if the ticks > myTimeOut then
    if myImmediateJump or (the frame = myEndFrame) then
      go(myTimeOutFrame)
    end if
  else
    if the frame = myEndFrame then
      go(myStartFrame)
    end if
  end if
end

on substituteStrings me, parentString, childStringList
  i = childStringList.count()
  repeat while i
    tempString = EMPTY
    dummyString = childStringList.getPropAt(i)
    replacement = childStringList[i]
    lengthAdjust = dummyString.char.count - 1
    repeat while 1
      position = offset(dummyString, parentString)
      if not position then
        parentString = tempString & parentString
        exit repeat
        next repeat
      end if
      if position <> 1 then
        tempString = tempString & parentString.char[1..position - 1]
      end if
      tempString = tempString & replacement
      delete me.char[1..position + lengthAdjust]
    end repeat
    i = i - 1
  end repeat
  return parentString
end

on ErrorAlert me, theError, data
  behaviorName = string(me)
  delete word 1 of behaviorName
  delete char -30001 of behaviorName
  delete char -30001 of behaviorName
  case theError of
    #invalidChannel:
      if the runMode = "Author" then
        terror1 = "BEHAVIOR ERROR: Frame ^0, Sprite ^1"
        terror1 = substituteStrings(me, terror1, ["^0": the frame, "^1": the currentSpriteNum])
        terror2 = "Behavior ^0 should be attached to the frame script channel."
        terror2 = substituteStrings(me, terror2, ["^0": behaviorName])
        terror3 = "Current Channel = ^0"
        terror3 = substituteStrings(me, terror3, ["^0": data])
        alert(terror1 & RETURN & RETURN & terror2 & RETURN & RETURN & terror3)
        abort()
      end if
    #missingMarker:
      if the runMode = "Author" then
        terror1 = "BEHAVIOR ERROR: Frame ^0, Sprite ^1"
        terror1 = substituteStrings(me, terror1, ["^0": the frame, "^1": the currentSpriteNum])
        terror2 = "Frame behavior ^0 is set to jump to marker ^1. " & "This marker cannot be found. " & " Choose a valid marker in the Behavior Parameters dialog."
        terror2 = substituteStrings(me, terror2, ["^0": behaviorName, "^1": myTimeOutFrame])
        terror3 = "In the meantime, frame ^0 will be used instead."
        terror3 = substituteStrings(me, terror3, ["^0": data])
        alert(terror1 & RETURN & RETURN & terror2 & RETURN & RETURN & terror3)
      end if
    #endlessLoop:
      if the runMode = "Author" then
        if symbolp(myTimeOutFrame) then
          case myTimeOutFrame of
            #previous:
              jumpToFrame = marker(-1)
            #loop:
              jumpToFrame = marker(0)
            #next:
              jumpToFrame = marker(1)
          end case
        else
          jumpToFrame = marker(myTimeOutFrame)
        end if
        terror1 = "BEHAVIOR ERROR: Frame ^0, Sprite ^1"
        terror1 = substituteStrings(me, terror1, ["^0": the frame, "^1": the currentSpriteNum])
        terror2 = "Frame behavior ^0 is set to jump to marker ^1 (frame ^2 ). " & "  This is within the span of the behavior and will cause an endless loop."
        terror2 = substituteStrings(me, terror2, ["^0": behaviorName, "^1": myTimeOutFrame, "^2": jumpToFrame])
        terror3 = "Frame ^0 will be used instead."
        terror3 = substituteStrings(me, terror3, ["^0": data])
        alert(terror1 & RETURN & RETURN & terror2 & RETURN & RETURN & terror3)
      end if
  end case
end

on isOKToAttach me, aSpriteType, aSpriteNum
  tisok = 0
  if aSpriteType = #script then
    tisok = 1
  end if
  return tisok
end

on getPropertyDescriptionList me
  nextMarker = nextMarker(me)
  return [#myTimeOut: [#comment: "Loop over selected frames for...", #format: #integer, #range: [#min: 1, #max: 120], #default: 30], #myTimeUnit: [#comment: EMPTY, #format: #string, #range: ["Ticks", "Seconds", "Minutes", "Hours"], #default: "Seconds"], #myTimeOutFrame: [#comment: "... " & "then jump to marker", #format: #marker, #default: nextMarker], #myImmediateJump: [#comment: "When the time is up", #format: #string, #range: ["complete the loop", "jump immediately"], #default: "jump immediately"]]
end

on nextMarker me
  labelString = the labelList
  delete char -30000 of labelString
  markerCount = the number of lines in labelString
  theFrame = the frame
  repeat with i = 1 to markerCount
    theMarker = line i of labelString
    markerFrame = marker(theMarker)
    if theFrame < markerFrame then
      return theMarker
    end if
  end repeat
end
